@@ -73,7 +73,7 @@ static NPClass _javascriptClass = {
 
 NPClass *NPScriptObjectClass = &_javascriptClass;
 
-Identifier identiferFromNPIdentifier(const NPUTF8 *name)
+static Identifier identifierFromNPIdentifier(const NPUTF8 *name)
 {
     NPUTF16 *methodName;
     unsigned int UTF16Length;
@@ -148,38 +148,29 @@ bool _NPN_Invoke (NPP npp, NPObject *o, NPIdentifier methodName, const NPVariant
 	else {
 	    // Lookup the function object.
 	    ExecState *exec = obj->executionContext->interpreter()->globalExec();
-	    Interpreter::lock();
-	    Value func = obj->imp->get (exec, identiferFromNPIdentifier(i->value.string));
-	    Interpreter::unlock();
+	    InterpreterLock lock;
+	    Value func = obj->imp->get (exec, identifierFromNPIdentifier(i->value.string));
 
 	    if (func.isNull()) {
 		NPN_InitializeVariantAsNull(result);
 		return false;
-	    }
-	    else if ( func.type() == UndefinedType) {
+	    } else if (func.type() == UndefinedType) {
 		NPN_InitializeVariantAsUndefined(result);
 		return false;
-	    }
-	    else {
+	    } else {
 		// Call the function object.
 		ObjectImp *funcImp = static_cast<ObjectImp*>(func.imp());
 		Object thisObj = Object(const_cast<ObjectImp*>(obj->imp));
 		List argList = listFromVariantArgs(exec, args, argCount);
-		Interpreter::lock();
-		Value resultV = funcImp->call (exec, thisObj, argList);
-		Interpreter::unlock();
+		Value resultV = Object(funcImp).call (exec, thisObj, argList);
 
 		// Convert and return the result of the function call.
 		convertValueToNPVariant(exec, resultV, result);
 		return true;
 	    }
 	}
-    }
-    else {
-        if (o->_class->invoke) {
-            return o->_class->invoke (o, methodName, args, argCount, result);
-        }
-    }
+    } else if (o->_class->invoke)
+        return o->_class->invoke (o, methodName, args, argCount, result);
     
     return true;
 }
@@ -196,7 +187,7 @@ bool _NPN_Evaluate (NPP npp, NPObject *o, NPString *s, NPVariant *variant)
         Object thisObj = Object(const_cast<ObjectImp*>(obj->imp));
         Value result;
         
-        Interpreter::lock();
+        InterpreterLock lock;
         NPUTF16 *scriptString;
         unsigned int UTF16Length;
         convertNPStringToUTF16 (s, &scriptString, &UTF16Length);    // requires free() of returned memory.
@@ -212,8 +203,6 @@ bool _NPN_Evaluate (NPP npp, NPObject *o, NPString *s, NPVariant *variant)
         else
             result = Undefined();
             
-        Interpreter::unlock();
-        
         free ((void *)scriptString);
         
         convertValueToNPVariant(exec, result, variant);
@@ -235,7 +224,7 @@ bool _NPN_GetProperty (NPP npp, NPObject *o, NPIdentifier propertyName, NPVarian
 
         PrivateIdentifier *i = (PrivateIdentifier *)propertyName;
         if (i->isString) {
-            if (!obj->imp->hasProperty (exec, identiferFromNPIdentifier(i->value.string))) {
+            if (!obj->imp->hasProperty (exec, identifierFromNPIdentifier(i->value.string))) {
                 NPN_InitializeVariantAsNull(variant);
                 return false;
             }
@@ -247,16 +236,13 @@ bool _NPN_GetProperty (NPP npp, NPObject *o, NPIdentifier propertyName, NPVarian
             }
         }
         
-        Interpreter::lock();
+        InterpreterLock lock;
         Value result;
-        if (i->isString) {
-            result = obj->imp->get (exec, identiferFromNPIdentifier(i->value.string));
-        }
-        else {
+        if (i->isString)
+            result = obj->imp->get (exec, identifierFromNPIdentifier(i->value.string));
+        else
             result = obj->imp->get (exec, i->value.number);
-        }
-        Interpreter::unlock();
-
+        
         if (result.isNull()) {
             NPN_InitializeVariantAsNull(variant);
             return false;
@@ -291,22 +277,17 @@ bool _NPN_SetProperty (NPP npp, NPObject *o, NPIdentifier propertyName, const NP
 	    return false;
 
         ExecState *exec = obj->executionContext->interpreter()->globalExec();
-        Interpreter::lock();
-        Value result;
+        InterpreterLock lock;
         PrivateIdentifier *i = (PrivateIdentifier *)propertyName;
-        if (i->isString) {
-            obj->imp->put (exec, identiferFromNPIdentifier(i->value.string), convertNPVariantToValue(exec, variant));
-        }
-        else {
-            obj->imp->put (exec, i->value.number, convertNPVariantToValue(exec, variant));
-        }
-        Interpreter::unlock();
+        if (i->isString)
+            obj->imp->put(exec, identifierFromNPIdentifier(i->value.string), convertNPVariantToValue(exec, variant));
+        else
+            obj->imp->put(exec, i->value.number, convertNPVariantToValue(exec, variant));
         
         return true;
-    }
-    else if (o->_class->setProperty) {
+    } else if (o->_class->setProperty)
         return o->_class->setProperty (o, propertyName, variant);
-    }
+
     return false;
 }
 
@@ -322,7 +303,7 @@ bool _NPN_RemoveProperty (NPP npp, NPObject *o, NPIdentifier propertyName)
 
         PrivateIdentifier *i = (PrivateIdentifier *)propertyName;
         if (i->isString) {
-            if (!obj->imp->hasProperty (exec, identiferFromNPIdentifier(i->value.string))) {
+            if (!obj->imp->hasProperty (exec, identifierFromNPIdentifier(i->value.string))) {
                 return false;
             }
         }
@@ -332,14 +313,11 @@ bool _NPN_RemoveProperty (NPP npp, NPObject *o, NPIdentifier propertyName)
             }
         }
 
-        Interpreter::lock();
-        if (i->isString) {
-            obj->imp->deleteProperty (exec, identiferFromNPIdentifier(i->value.string));
-        }
-        else {
+        InterpreterLock lock;
+        if (i->isString)
+            obj->imp->deleteProperty (exec, identifierFromNPIdentifier(i->value.string));
+        else
             obj->imp->deleteProperty (exec, i->value.number);
-        }
-        Interpreter::unlock();
         
         return true;
     }
@@ -357,24 +335,14 @@ bool _NPN_HasProperty(NPP npp, NPObject *o, NPIdentifier propertyName)
         ExecState *exec = obj->executionContext->interpreter()->globalExec();
 
         PrivateIdentifier *i = (PrivateIdentifier *)propertyName;
-        // String identifier?
-        if (i->isString) {
-            ExecState *exec = obj->executionContext->interpreter()->globalExec();
-            Interpreter::lock();
-            bool result = obj->imp->hasProperty (exec, identiferFromNPIdentifier(i->value.string));
-            Interpreter::unlock();
-            return result;
-        }
+        InterpreterLock lock;
+
+        if (i->isString)
+            return obj->imp->hasProperty(exec, identifierFromNPIdentifier(i->value.string));
         
-        // Numeric identifer
-        Interpreter::lock();
-        bool result = obj->imp->hasProperty (exec, i->value.number);
-        Interpreter::unlock();
-        return result;
-    }
-    else if (o->_class->hasProperty) {
+        return obj->imp->hasProperty(exec, i->value.number);
+    } else if (o->_class->hasProperty)
         return o->_class->hasProperty (o, propertyName);
-    }
     
     return false;
 }
@@ -393,9 +361,9 @@ bool _NPN_HasMethod(NPP npp, NPObject *o, NPIdentifier methodName)
             
         // Lookup the function object.
         ExecState *exec = obj->executionContext->interpreter()->globalExec();
-        Interpreter::lock();
-        Value func = obj->imp->get (exec, identiferFromNPIdentifier(i->value.string));
-        Interpreter::unlock();
+
+        InterpreterLock lock;
+        Value func = obj->imp->get (exec, identifierFromNPIdentifier(i->value.string));
 
         if (func.isNull() || func.type() == UndefinedType) {
             return false;
@@ -416,9 +384,8 @@ void _NPN_SetException (NPObject *o, const NPUTF8 *message)
     if (o->_class == NPScriptObjectClass) {
         JavaScriptObject *obj = (JavaScriptObject *)o; 
         ExecState *exec = obj->executionContext->interpreter()->globalExec();
-        Interpreter::lock();
+        InterpreterLock lock;
         Object err = Error::create(exec, GeneralError, message);
         exec->setException (err);
-        Interpreter::unlock();
     }
 }
