@@ -32,6 +32,7 @@
 #include <runtime.h>
 #include <ustring.h>
 
+
 namespace KJS
 {
 class Value;
@@ -190,6 +191,9 @@ public:
     JNIType getJNIType() const { return _JNIType; }
     
 private:
+    void JavaField::dispatchSetValueToInstance(KJS::ExecState *exec, const JavaInstance *instance, jvalue javaValue, const char *name, const char *sig) const;
+    jvalue JavaField::dispatchValueFromInstance(KJS::ExecState *exec, const JavaInstance *instance, const char *name, const char *sig, JNIType returnType) const;
+
     JavaString _name;
     JavaString _type;
     JNIType _JNIType;
@@ -250,6 +254,8 @@ public:
     JNIType JNIReturnType() const;
 
     jmethodID methodID (jobject obj) const;
+    
+    bool isStatic() const { return _isStatic; }
 	
 private:
     JavaParameter *_parameters;
@@ -259,12 +265,13 @@ private:
     JavaString _returnType;
     JNIType _JNIReturnType;
     mutable jmethodID _methodID;
+    bool _isStatic;
 };
 
 class JavaArray : public Array
 {
 public:
-    JavaArray (jobject a, const char *type);
+    JavaArray (jobject a, const char *type, const RootObject *r);
 
     JavaArray (const JavaArray &other);
 
@@ -274,6 +281,7 @@ public:
         
         free ((void *)_type);
         _type = strdup(other._type);
+        _root = other._root;
 
         JObjectWrapper *_oldArray = _array;
         _array = other._array;
@@ -291,12 +299,15 @@ public:
 
     jobject javaArray() const { return _array->_instance; }
 
-    static KJS::Value convertJObjectToArray (KJS::ExecState *exec, jobject anObject, const char *type);
+    static KJS::Value convertJObjectToArray (KJS::ExecState *exec, jobject anObject, const char *type, const RootObject *r);
 
+    const RootObject *executionContext() const { return _root; }
+    
 private:
     JObjectWrapper *_array;
     unsigned int _length;
     const char *_type;
+    const RootObject *_root;
 };
 
 } // namespace Bindings
