@@ -138,7 +138,7 @@ void ArrayInstanceImp::put(ExecState *exec, unsigned index, const Value &value,
   ObjectImp::put(exec, Identifier::from(index), value, attr);
 }
 
-bool ArrayInstanceImp::hasProperty(ExecState *exec, const Identifier &propertyName) const
+bool ArrayInstanceImp::hasOwnProperty(ExecState *exec, const Identifier &propertyName) const
 {
   if (propertyName == lengthPropertyName)
     return true;
@@ -154,10 +154,10 @@ bool ArrayInstanceImp::hasProperty(ExecState *exec, const Identifier &propertyNa
     }
   }
   
-  return ObjectImp::hasProperty(exec, propertyName);
+  return ObjectImp::hasOwnProperty(exec, propertyName);
 }
 
-bool ArrayInstanceImp::hasProperty(ExecState *exec, unsigned index) const
+bool ArrayInstanceImp::hasOwnProperty(ExecState *exec, unsigned index) const
 {
   if (index >= length)
     return false;
@@ -166,7 +166,7 @@ bool ArrayInstanceImp::hasProperty(ExecState *exec, unsigned index) const
     return v && v != UndefinedImp::staticUndefined;
   }
   
-  return ObjectImp::hasProperty(exec, Identifier::from(index));
+  return ObjectImp::hasOwnProperty(exec, Identifier::from(index));
 }
 
 bool ArrayInstanceImp::deleteProperty(ExecState *exec, const Identifier &propertyName)
@@ -461,7 +461,7 @@ Value ArrayProtoFuncImp::call(ExecState *exec, Object &thisObj, const List &args
     UString separator = ",";
     UString str = "";
 
-    if (args.size() > 0)
+    if (args[0].type() != UndefinedType)
       separator = args[0].toString(exec);
     for (unsigned int k = 0; k < length; k++) {
       if (k >= 1)
@@ -580,24 +580,27 @@ Value ArrayProtoFuncImp::call(ExecState *exec, Object &thisObj, const List &args
     // We return a new array
     Object resObj = Object::dynamicCast(exec->lexicalInterpreter()->builtinArray().construct(exec,List::empty()));
     result = resObj;
-    double begin = args[0].toInteger(exec);
-    if (begin < 0) {
-      begin += length;
-      if (begin < 0)
-        begin = 0;
-    } else {
-      if (begin > length)
-        begin = length;
+    double begin = 0;
+    if (args[0].type() != UndefinedType) {
+        begin = args[0].toInteger(exec);
+        if (begin >= 0) { // false for NaN
+            if (begin > length)
+                begin = length;
+        } else {
+            begin += length;
+            if (!(begin >= 0)) // true for NaN
+                begin = 0;
+        }
     }
     double end = length;
     if (args[1].type() != UndefinedType) {
       end = args[1].toInteger(exec);
-      if (end < 0) {
+      if (end < 0) { // false for NaN
         end += length;
         if (end < 0)
           end = 0;
       } else {
-        if (end > length)
+        if (!(end <= length)) // true for NaN
           end = length;
       }
     }
