@@ -21,19 +21,14 @@
 
 #include "collector.h"
 
-#include "value.h"
+#include "fast_malloc.h"
 #include "internal.h"
+#include "list.h"
+#include "value.h"
 
 #if APPLE_CHANGES
 #include <CoreFoundation/CoreFoundation.h>
 #include <cxxabi.h>
-#endif
-
-#include <collector.h>
-#include <value.h>
-#include <internal.h>
-
-#if APPLE_CHANGES
 #include <pthread.h>
 #include <mach/mach_port.h>
 #include <mach/task.h>
@@ -63,7 +58,7 @@ struct CollectorCell {
     double memory[CELL_ARRAY_LENGTH];
     struct {
       void *zeroIfFree;
-      CollectorCell *next;
+      ptrdiff_t next;
     } freeCell;
   } u;
 };
@@ -86,7 +81,7 @@ struct CollectorHeap {
   int usedOversizeCells;
 
   int numLiveObjects;
-  int numAllocationsSinceLastCollect;
+  int numLiveObjectsAtLastCollect;
 };
 
 static CollectorHeap heap = {NULL, 0, 0, 0, NULL, 0, 0, 0, 0};
@@ -97,29 +92,29 @@ void* Collector::allocate(size_t s)
 {
   assert(Interpreter::lockCount() > 0);
 
-  if (s == 0)
-    return 0L;
-  
   // collect if needed
-  if (++heap.numAllocationsSinceLastCollect >= ALLOCATIONS_PER_COLLECTION) {
+  int numLiveObjects = heap.numLiveObjects;
+  if (numLiveObjects - heap.numLiveObjectsAtLastCollect >= ALLOCATIONS_PER_COLLECTION) {
     collect();
+    numLiveObjects = heap.numLiveObjects;
   }
   
-  if (s > (unsigned)CELL_SIZE) {
+  if (s > static_cast<size_t>(CELL_SIZE)) {
     // oversize allocator
     if (heap.usedOversizeCells == heap.numOversizeCells) {
       heap.numOversizeCells = MAX(MIN_ARRAY_SIZE, heap.numOversizeCells * GROWTH_FACTOR);
-      heap.oversizeCells = (CollectorCell **)realloc(heap.oversizeCells, heap.numOversizeCells * sizeof(CollectorCell *));
+      heap.oversizeCells = (CollectorCell **)kjs_fast_realloc(heap.oversizeCells, heap.numOversizeCells * sizeof(CollectorCell *));
     }
     
-    void *newCell = malloc(s);
+    void *newCell = kjs_fast_malloc(s);
     heap.oversizeCells[heap.usedOversizeCells] = (CollectorCell *)newCell;
     heap.usedOversizeCells++;
-    heap.numLiveObjects++;
+    heap.numLiveObjects = numLiveObjects + 1;
 
 #if !USE_CONSERVATIVE_GC
     ((ValueImp *)(newCell))->_flags = 0;
 #endif
+
     return newCell;
   }
   
@@ -142,10 +137,10 @@ void* Collector::allocate(size_t s)
     
     if (heap.usedBlocks == heap.numBlocks) {
       heap.numBlocks = MAX(MIN_ARRAY_SIZE, heap.numBlocks * GROWTH_FACTOR);
-      heap.blocks = (CollectorBlock **)realloc(heap.blocks, heap.numBlocks * sizeof(CollectorBlock *));
+      heap.blocks = (CollectorBlock **)kjs_fast_realloc(heap.blocks, heap.numBlocks * sizeof(CollectorBlock *));
     }
     
-    targetBlock = (CollectorBlock *)calloc(1, sizeof(CollectorBlock));
+    targetBlock = (CollectorBlock *)kjs_fast_calloc(1, sizeof(CollectorBlock));
     targetBlock->freeList = targetBlock->cells;
     heap.blocks[heap.usedBlocks] = targetBlock;
     heap.usedBlocks++;
@@ -154,23 +149,18 @@ void* Collector::allocate(size_t s)
   // find a free spot in the block and detach it from the free list
   CollectorCell *newCell = targetBlock->freeList;
 
-  if (newCell->u.freeCell.next != NULL) {
-    targetBlock->freeList = newCell->u.freeCell.next;
-  } else if (targetBlock->usedCells == (CELLS_PER_BLOCK - 1)) {
-    // last cell in this block
-    targetBlock->freeList = NULL;
-  } else {
-    // all next pointers are initially 0, meaning "next cell"
-    targetBlock->freeList = newCell + 1;
-  }
+  // "next" field is a byte offset -- 0 means next cell, so a zeroed block is already initialized
+  // could avoid the casts by using a cell offset, but this avoids a relatively-slow multiply
+  targetBlock->freeList = reinterpret_cast<CollectorCell *>(reinterpret_cast<char *>(newCell + 1) + newCell->u.freeCell.next);
 
   targetBlock->usedCells++;
-  heap.numLiveObjects++;
+  heap.numLiveObjects = numLiveObjects + 1;
 
 #if !USE_CONSERVATIVE_GC
   ((ValueImp *)(newCell))->_flags = 0;
 #endif
-  return (void *)(newCell);
+
+  return newCell;
 }
 
 #if TEST_CONSERVATIVE_GC || USE_CONSERVATIVE_GC
@@ -224,9 +214,10 @@ void Collector::registerThread()
   }
 }
 
+#define IS_POINTER_ALIGNED(p) (((int)(p) & (sizeof(char *) - 1)) == 0)
 
-// cells are 8-byte aligned 
-#define IS_POINTER_ALIGNED(p) (((int)(p) & 7) == 0)
+// cells are 8-byte aligned
+#define IS_CELL_ALIGNED(p) (((int)(p) & 7) == 0)
 
 void Collector::markStackObjectsConservatively(void *start, void *end)
 {
@@ -245,7 +236,7 @@ void Collector::markStackObjectsConservatively(void *start, void *end)
   
   while (p != e) {
     char *x = *p++;
-    if (IS_POINTER_ALIGNED(x) && x) {
+    if (IS_CELL_ALIGNED(x) && x) {
       bool good = false;
       for (int block = 0; block < heap.usedBlocks; block++) {
 	size_t offset = x - (char *)heap.blocks[block];
@@ -339,8 +330,10 @@ void Collector::markStackObjectsConservatively()
 
 void Collector::markProtectedObjects()
 {
-  for (int i = 0; i < ProtectedValues::_tableSize; i++) {
-    ValueImp *val = ProtectedValues::_table[i].key;
+  int size = ProtectedValues::_tableSize;
+  ProtectedValues::KeyValue *table = ProtectedValues::_table;
+  for (int i = 0; i < size; i++) {
+    ValueImp *val = table[i].key;
     if (val && !val->marked()) {
       val->mark();
     }
@@ -372,6 +365,7 @@ bool Collector::collect()
 
   markStackObjectsConservatively();
   markProtectedObjects();
+  List::markProtectedLists();
 #endif
 
 #if TEST_CONSERVATIVE_GC
@@ -428,20 +422,22 @@ bool Collector::collect()
   // SWEEP: delete everything with a zero refcount (garbage) and unmark everything else
   
   int emptyBlocks = 0;
+  int numLiveObjects = heap.numLiveObjects;
 
   for (int block = 0; block < heap.usedBlocks; block++) {
     CollectorBlock *curBlock = heap.blocks[block];
 
     int minimumCellsToProcess = curBlock->usedCells;
 
-    for (int cell = 0; cell < CELLS_PER_BLOCK; cell++) {
-      if (minimumCellsToProcess < cell) {
+    for (int i = 0; i < CELLS_PER_BLOCK; i++) {
+      if (minimumCellsToProcess < i) {
 	goto skip_block_sweep;
       }
 
-      ValueImp *imp = (ValueImp *)(curBlock->cells + cell);
+      CollectorCell *cell = curBlock->cells + i;
+      ValueImp *imp = reinterpret_cast<ValueImp *>(cell);
 
-      if (((CollectorCell *)imp)->u.freeCell.zeroIfFree != 0) {
+      if (cell->u.freeCell.zeroIfFree != 0) {
 #if USE_CONSERVATIVE_GC
 	if (!imp->_marked)
 #else
@@ -452,13 +448,13 @@ bool Collector::collect()
 	  // emulate destructing part of 'operator delete()'
 	  imp->~ValueImp();
 	  curBlock->usedCells--;
-	  heap.numLiveObjects--;
+	  numLiveObjects--;
 	  deleted = true;
 
 	  // put it on the free list
-	  ((CollectorCell *)imp)->u.freeCell.zeroIfFree = 0;
-	  ((CollectorCell *)imp)->u.freeCell.next = curBlock->freeList;
-	  curBlock->freeList = (CollectorCell *)imp;
+	  cell->u.freeCell.zeroIfFree = 0;
+	  cell->u.freeCell.next = reinterpret_cast<char *>(curBlock->freeList) - reinterpret_cast<char *>(cell + 1);
+	  curBlock->freeList = cell;
 
 	} else {
 #if USE_CONSERVATIVE_GC
@@ -480,7 +476,7 @@ bool Collector::collect()
       emptyBlocks++;
       if (emptyBlocks > SPARE_EMPTY_BLOCKS) {
 #if !DEBUG_COLLECTOR
-	free(heap.blocks[block]);
+	kjs_fast_free(heap.blocks[block]);
 #endif
 	// swap with the last block so we compact as we go
 	heap.blocks[block] = heap.blocks[heap.usedBlocks - 1];
@@ -489,7 +485,7 @@ bool Collector::collect()
 
 	if (heap.numBlocks > MIN_ARRAY_SIZE && heap.usedBlocks < heap.numBlocks / LOW_WATER_FACTOR) {
 	  heap.numBlocks = heap.numBlocks / GROWTH_FACTOR; 
-	  heap.blocks = (CollectorBlock **)realloc(heap.blocks, heap.numBlocks * sizeof(CollectorBlock *));
+	  heap.blocks = (CollectorBlock **)kjs_fast_realloc(heap.blocks, heap.numBlocks * sizeof(CollectorBlock *));
 	}
       } 
     }
@@ -514,7 +510,7 @@ bool Collector::collect()
 #if DEBUG_COLLECTOR
       heap.oversizeCells[cell]->u.freeCell.zeroIfFree = 0;
 #else
-      free((void *)imp);
+      kjs_fast_free((void *)imp);
 #endif
 
       // swap with the last oversize cell so we compact as we go
@@ -522,11 +518,11 @@ bool Collector::collect()
 
       heap.usedOversizeCells--;
       deleted = true;
-      heap.numLiveObjects--;
+      numLiveObjects--;
 
       if (heap.numOversizeCells > MIN_ARRAY_SIZE && heap.usedOversizeCells < heap.numOversizeCells / LOW_WATER_FACTOR) {
 	heap.numOversizeCells = heap.numOversizeCells / GROWTH_FACTOR; 
-	heap.oversizeCells = (CollectorCell **)realloc(heap.oversizeCells, heap.numOversizeCells * sizeof(CollectorCell *));
+	heap.oversizeCells = (CollectorCell **)kjs_fast_realloc(heap.oversizeCells, heap.numOversizeCells * sizeof(CollectorCell *));
       }
 
     } else {
@@ -541,9 +537,10 @@ bool Collector::collect()
     }
   }
   
-  heap.numAllocationsSinceLastCollect = 0;
+  heap.numLiveObjects = numLiveObjects;
+  heap.numLiveObjectsAtLastCollect = numLiveObjects;
   
-  memoryFull = (heap.numLiveObjects >= KJS_MEM_LIMIT);
+  memoryFull = (numLiveObjects >= KJS_MEM_LIMIT);
 
   return deleted;
 }
@@ -607,8 +604,10 @@ int Collector::numReferencedObjects()
   int count = 0;
 
 #if USE_CONSERVATIVE_GC
-  for (int i = 0; i < ProtectedValues::_tableSize; i++) {
-    ValueImp *val = ProtectedValues::_table[i].key;
+  int size = ProtectedValues::_tableSize;
+  ProtectedValues::KeyValue *table = ProtectedValues::_table;
+  for (int i = 0; i < size; i++) {
+    ValueImp *val = table[i].key;
     if (val) {
       ++count;
     }
@@ -645,8 +644,10 @@ const void *Collector::rootObjectClasses()
   CFMutableSetRef classes = CFSetCreateMutable(NULL, 0, &kCFTypeSetCallBacks);
 
 #if USE_CONSERVATIVE_GC
-  for (int i = 0; i < ProtectedValues::_tableSize; i++) {
-    ValueImp *val = ProtectedValues::_table[i].key;
+  int size = ProtectedValues::_tableSize;
+  ProtectedValues::KeyValue *table = ProtectedValues::_table;
+  for (int i = 0; i < size; i++) {
+    ValueImp *val = table[i].key;
     if (val) {
       const char *mangled_name = typeid(*val).name();
       int status;
