@@ -20,21 +20,21 @@
  * 
  */
 /**
-    File Name:          15.9.5.16.js
-    ECMA Section:       15.9.5.16
-    Description:        Date.prototype.getMinutes
-    1.Let t be this time value.
-    2.If t is NaN, return NaN.
-    3.Return MinFromTime(LocalTime(t)).
+    File Name:          15.9.5.6.js
+    ECMA Section:       15.9.5.6
+    Description:        Date.prototype.getFullYear
 
+   1.   Let t be this time value.
+   2.   If t is NaN, return NaN.
+   3.   Return YearFromTime(LocalTime(t)).
     Author:             christine@netscape.com
     Date:               12 november 1997
 */
 
-    var SECTION = "15.9.5.16";
+    var SECTION = "15.9.5.6";
     var VERSION = "ECMA_1";
     startTest();
-    var TITLE   = "Date.prototype.getMinutes()";
+    var TITLE   = "Date.prototype.getFullYear()";
 
     writeHeaderToLog( SECTION + " "+ TITLE);
 
@@ -67,23 +67,41 @@
     addTestCase( UTC_JAN_1_2005 );
 
     testcases[tc++] = new TestCase( SECTION,
-                                    "(new Date(NaN)).getMinutes()",
+                                    "(new Date(NaN)).getFullYear()",
                                     NaN,
-                                    (new Date(NaN)).getMinutes() );
+                                    (new Date(NaN)).getFullYear() );
 
     testcases[tc++] = new TestCase( SECTION,
-                                    "Date.prototype.getMinutes.length",
+                                    "Date.prototype.getFullYear.length",
                                     0,
-                                    Date.prototype.getMinutes.length );
+                                    Date.prototype.getFullYear.length );
+
     test();
 function addTestCase( t ) {
-    for ( m = 0; m <= 60; m+=10 ) {
-            t += msPerMinute;
-            testcases[tc++] = new TestCase( SECTION,
-                                    "(new Date("+t+")).getMinutes()",
-                                    MinFromTime((LocalTime(t))),
-                                    (new Date(t)).getMinutes() );
-    }
+    testcases[tc++] = new TestCase( SECTION,
+                                    "(new Date("+t+")).getFullYear()",
+                                    YearFromTime(LocalTime(t)),
+                                    (new Date(t)).getFullYear() );
+
+    testcases[tc++] = new TestCase( SECTION,
+                                    "(new Date("+(t+1)+")).getFullYear()",
+                                    YearFromTime(LocalTime(t+1)),
+                                    (new Date(t+1)).getFullYear() );
+
+    testcases[tc++] = new TestCase( SECTION,
+                                    "(new Date("+(t-1)+")).getFullYear()",
+                                    YearFromTime(LocalTime(t-1)),
+                                    (new Date(t-1)).getFullYear() );
+
+    testcases[tc++] = new TestCase( SECTION,
+                                    "(new Date("+(t-TZ_ADJUST)+")).getFullYear()",
+                                    YearFromTime(LocalTime(t-TZ_ADJUST)),
+                                    (new Date(t-TZ_ADJUST)).getFullYear() );
+
+    testcases[tc++] = new TestCase( SECTION,
+                                    "(new Date("+(t+TZ_ADJUST)+")).getFullYear()",
+                                    YearFromTime(LocalTime(t+TZ_ADJUST)),
+                                    (new Date(t+TZ_ADJUST)).getFullYear() );
 }
 function test() {
     for ( tc=0; tc < testcases.length; tc++ ) {
